/*******************************
 * GConfDialog
 * Author: Luca Viggiani
 * Version: 0.1
 * Date: 2009-12-07
 * License: GPL
 * Description: creates a gconf settings back end dialog 
 * 				automatically from a settings key root.
 * Usage: new GConfDialog (string key, string dialog_title)
 * Params:
 * 	    string key: the gconf key to connect to (i.e /apps/panel/applets/applet_46/prefs)
 * 		string dislog_title: text to be shown on window title bar
 *		string[] subkeys: used to choose the subkeys to hadle (comprising display order).
 * 			  if null all subkeys will be retieved.
 * 
 * TODO: Only BOOL and INT data types are implemented. STRING is implemented only partially (read)
 *******************************/

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gconf/gconf-client.h>
#include <stdlib.h>
#include <string.h>
#include <pango/pango.h>
#include <gconf/gconf.h>
#include <glib/gi18n-lib.h>


#define TYPE_GCONF_DIALOG (gconf_dialog_get_type ())
#define GCONF_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GCONF_DIALOG, GConfDialog))
#define GCONF_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GCONF_DIALOG, GConfDialogClass))
#define IS_GCONF_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GCONF_DIALOG))
#define IS_GCONF_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GCONF_DIALOG))
#define GCONF_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GCONF_DIALOG, GConfDialogClass))

typedef struct _GConfDialog GConfDialog;
typedef struct _GConfDialogClass GConfDialogClass;
typedef struct _GConfDialogPrivate GConfDialogPrivate;

struct _GConfDialog {
	GtkDialog parent_instance;
	GConfDialogPrivate * priv;
};

struct _GConfDialogClass {
	GtkDialogClass parent_class;
};

struct _GConfDialogPrivate {
	GConfClient* _default_client;
};



GType gconf_dialog_get_type (void);
#define GCONF_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_GCONF_DIALOG, GConfDialogPrivate))
enum  {
	GCONF_DIALOG_DUMMY_PROPERTY
};
GConfDialog* gconf_dialog_new (const char* dialog_title);
GConfDialog* gconf_dialog_construct (GType object_type, const char* dialog_title);
GConfDialog* gconf_dialog_new (const char* dialog_title);
static GtkWidget* gconf_dialog_create_proxy_widget (GConfDialog* self, const char* key, GError** error);
void gconf_dialog_add_key_group (GConfDialog* self, const char* group_name, char** keys, int keys_length1);
static GConfSchema* gconf_dialog_safely_get_schema (GConfDialog* self, GConfEntry* entry);
static void gconf_dialog_onCheckButtonActivated (GConfDialog* self, GtkCheckButton* widget);
static void _gconf_dialog_onCheckButtonActivated_gtk_button_clicked (GtkCheckButton* _sender, gpointer self);
static void gconf_dialog_onSpinButtonValueChanged (GConfDialog* self, GtkSpinButton* widget);
static void _gconf_dialog_onSpinButtonValueChanged_gtk_spin_button_value_changed (GtkSpinButton* _sender, gpointer self);
static void gconf_dialog_onResetButtonPressed (GConfDialog* self, GtkButton* widget);
static void _gconf_dialog_onResetButtonPressed_gtk_button_clicked (GtkButton* _sender, gpointer self);
static GObject * gconf_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer gconf_dialog_parent_class = NULL;
static void gconf_dialog_finalize (GObject* obj);



GConfDialog* gconf_dialog_construct (GType object_type, const char* dialog_title) {
	GConfDialog * self;
	g_return_val_if_fail (dialog_title != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	gtk_window_set_title ((GtkWindow*) self, dialog_title);
	gtk_window_set_icon_name ((GtkWindow*) self, "gtk-preferences");
	return self;
}


GConfDialog* gconf_dialog_new (const char* dialog_title) {
	return gconf_dialog_construct (TYPE_GCONF_DIALOG, dialog_title);
}


void gconf_dialog_add_key_group (GConfDialog* self, const char* group_name, char** keys, int keys_length1) {
	GError * _inner_error_;
	GtkFrame* frame;
	GtkAlignment* alignment;
	GtkVBox* box;
	GtkWidget* _tmp0_;
	GtkWidget* label_widget;
	PangoFontDescription* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (group_name != NULL);
	_inner_error_ = NULL;
	frame = g_object_ref_sink ((GtkFrame*) gtk_frame_new (group_name));
	alignment = g_object_ref_sink ((GtkAlignment*) gtk_alignment_new (0.0f, 0.0f, 1.0f, 1.0f));
	box = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0));
	_tmp0_ = NULL;
	label_widget = (_tmp0_ = gtk_frame_get_label_widget (frame), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	_tmp1_ = NULL;
	gtk_widget_modify_font (label_widget, _tmp1_ = pango_font_description_from_string ("Bold"));
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (pango_font_description_free (_tmp1_), NULL));
	gtk_container_set_border_width ((GtkContainer*) frame, (guint) 10);
	gtk_container_add ((GtkContainer*) frame, (GtkWidget*) alignment);
	gtk_container_add ((GtkContainer*) alignment, (GtkWidget*) box);
	g_object_set (alignment, "left-padding", (guint) 10, NULL);
	{
		{
			char** key_collection;
			int key_collection_length1;
			int key_it;
			key_collection = keys;
			key_collection_length1 = keys_length1;
			for (key_it = 0; key_it < keys_length1; key_it = key_it + 1) {
				const char* key;
				key = key_collection[key_it];
				{
					GtkWidget* widget;
					widget = gconf_dialog_create_proxy_widget (self, key, &_inner_error_);
					if (_inner_error_ != NULL) {
						goto __catch6_g_error;
						goto __finally6;
					}
					gtk_box_pack_start ((GtkBox*) box, widget, FALSE, FALSE, (guint) 5);
					(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
				}
			}
		}
	}
	goto __finally6;
	__catch6_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			g_warning ("gtkextra-gconfdialog.vala:55: %s", e->message);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally6:
	if (_inner_error_ != NULL) {
		(frame == NULL) ? NULL : (frame = (g_object_unref (frame), NULL));
		(alignment == NULL) ? NULL : (alignment = (g_object_unref (alignment), NULL));
		(box == NULL) ? NULL : (box = (g_object_unref (box), NULL));
		(label_widget == NULL) ? NULL : (label_widget = (g_object_unref (label_widget), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	gtk_box_pack_start ((GtkBox*) ((GtkDialog*) self)->vbox, (GtkWidget*) frame, TRUE, FALSE, (guint) 0);
	gtk_widget_show_all ((GtkWidget*) frame);
	(frame == NULL) ? NULL : (frame = (g_object_unref (frame), NULL));
	(alignment == NULL) ? NULL : (alignment = (g_object_unref (alignment), NULL));
	(box == NULL) ? NULL : (box = (g_object_unref (box), NULL));
	(label_widget == NULL) ? NULL : (label_widget = (g_object_unref (label_widget), NULL));
}


static void _gconf_dialog_onCheckButtonActivated_gtk_button_clicked (GtkCheckButton* _sender, gpointer self) {
	gconf_dialog_onCheckButtonActivated (self, _sender);
}


static void _gconf_dialog_onSpinButtonValueChanged_gtk_spin_button_value_changed (GtkSpinButton* _sender, gpointer self) {
	gconf_dialog_onSpinButtonValueChanged (self, _sender);
}


static void _gconf_dialog_onResetButtonPressed_gtk_button_clicked (GtkButton* _sender, gpointer self) {
	gconf_dialog_onResetButtonPressed (self, _sender);
}


static GtkWidget* gconf_dialog_create_proxy_widget (GConfDialog* self, const char* key, GError** error) {
	GError * _inner_error_;
	GConfEntry* _tmp0_;
	GConfEntry* entry;
	GConfSchema* schema;
	const char* tooltip;
	GtkBox* row;
	GtkLabel* label;
	GtkWidget* action_widget;
	GtkWidget* render_widget;
	GtkButton* reset;
	GtkWidget* _tmp17_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (key != NULL, NULL);
	_inner_error_ = NULL;
	_tmp0_ = NULL;
	entry = (_tmp0_ = gconf_client_get_entry (self->priv->_default_client, key, NULL, TRUE, &_inner_error_), (_tmp0_ == NULL) ? NULL : gconf_entry_ref (_tmp0_), _tmp0_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
	schema = gconf_dialog_safely_get_schema (self, entry);
	tooltip = gconf_schema_get_long_desc (schema);
	row = (GtkBox*) g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 0));
	label = g_object_ref_sink ((GtkLabel*) gtk_label_new (gconf_schema_get_short_desc (schema)));
	gtk_label_set_justify (label, GTK_JUSTIFY_LEFT);
	gtk_widget_set_tooltip_text ((GtkWidget*) label, tooltip);
	action_widget = NULL;
	render_widget = NULL;
	switch (gconf_schema_get_type (schema)) {
		case GCONF_VALUE_BOOL:
		{
			GtkCheckButton* checkbox;
			gboolean _tmp1_;
			GtkWidget* _tmp3_;
			GtkWidget* _tmp2_;
			GtkWidget* _tmp5_;
			GtkWidget* _tmp4_;
			checkbox = g_object_ref_sink ((GtkCheckButton*) gtk_check_button_new ());
			_tmp1_ = gconf_client_get_bool (self->priv->_default_client, entry->key, &_inner_error_);
			if (_inner_error_ != NULL) {
				g_propagate_error (error, _inner_error_);
				(checkbox == NULL) ? NULL : (checkbox = (g_object_unref (checkbox), NULL));
				(entry == NULL) ? NULL : (entry = (gconf_entry_unref (entry), NULL));
				(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
				(row == NULL) ? NULL : (row = (g_object_unref (row), NULL));
				(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
				(action_widget == NULL) ? NULL : (action_widget = (g_object_unref (action_widget), NULL));
				(render_widget == NULL) ? NULL : (render_widget = (g_object_unref (render_widget), NULL));
				return NULL;
			}
			gtk_toggle_button_set_active ((GtkToggleButton*) checkbox, _tmp1_);
			g_signal_connect_object ((GtkButton*) checkbox, "clicked", (GCallback) _gconf_dialog_onCheckButtonActivated_gtk_button_clicked, self, 0);
			gtk_container_add ((GtkContainer*) checkbox, (GtkWidget*) label);
			_tmp3_ = NULL;
			_tmp2_ = NULL;
			render_widget = (_tmp3_ = (_tmp2_ = (GtkWidget*) checkbox, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (render_widget == NULL) ? NULL : (render_widget = (g_object_unref (render_widget), NULL)), _tmp3_);
			_tmp5_ = NULL;
			_tmp4_ = NULL;
			action_widget = (_tmp5_ = (_tmp4_ = render_widget, (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_)), (action_widget == NULL) ? NULL : (action_widget = (g_object_unref (action_widget), NULL)), _tmp5_);
			(checkbox == NULL) ? NULL : (checkbox = (g_object_unref (checkbox), NULL));
			break;
		}
		case GCONF_VALUE_STRING:
		{
			GtkEntry* entrybox;
			GtkHBox* box;
			const char* _tmp6_;
			GtkWidget* _tmp8_;
			GtkWidget* _tmp7_;
			GtkWidget* _tmp10_;
			GtkWidget* _tmp9_;
			entrybox = g_object_ref_sink ((GtkEntry*) gtk_entry_new ());
			box = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 0));
			_tmp6_ = gconf_client_get_string (self->priv->_default_client, entry->key, &_inner_error_);
			if (_inner_error_ != NULL) {
				g_propagate_error (error, _inner_error_);
				(entrybox == NULL) ? NULL : (entrybox = (g_object_unref (entrybox), NULL));
				(box == NULL) ? NULL : (box = (g_object_unref (box), NULL));
				(entry == NULL) ? NULL : (entry = (gconf_entry_unref (entry), NULL));
				(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
				(row == NULL) ? NULL : (row = (g_object_unref (row), NULL));
				(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
				(action_widget == NULL) ? NULL : (action_widget = (g_object_unref (action_widget), NULL));
				(render_widget == NULL) ? NULL : (render_widget = (g_object_unref (render_widget), NULL));
				return NULL;
			}
			gtk_entry_set_text (entrybox, _tmp6_);
			gtk_box_pack_start ((GtkBox*) box, (GtkWidget*) label, FALSE, FALSE, (guint) 2);
			gtk_box_pack_start ((GtkBox*) box, (GtkWidget*) entrybox, FALSE, FALSE, (guint) 2);
			_tmp8_ = NULL;
			_tmp7_ = NULL;
			action_widget = (_tmp8_ = (_tmp7_ = (GtkWidget*) entrybox, (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_)), (action_widget == NULL) ? NULL : (action_widget = (g_object_unref (action_widget), NULL)), _tmp8_);
			_tmp10_ = NULL;
			_tmp9_ = NULL;
			render_widget = (_tmp10_ = (_tmp9_ = (GtkWidget*) box, (_tmp9_ == NULL) ? NULL : g_object_ref (_tmp9_)), (render_widget == NULL) ? NULL : (render_widget = (g_object_unref (render_widget), NULL)), _tmp10_);
			(entrybox == NULL) ? NULL : (entrybox = (g_object_unref (entrybox), NULL));
			(box == NULL) ? NULL : (box = (g_object_unref (box), NULL));
			break;
		}
		case GCONF_VALUE_INT:
		{
			GtkHBox* box;
			GtkSpinButton* spin;
			gint _tmp11_;
			GtkWidget* _tmp13_;
			GtkWidget* _tmp12_;
			GtkWidget* _tmp15_;
			GtkWidget* _tmp14_;
			box = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 0));
			spin = g_object_ref_sink ((GtkSpinButton*) gtk_spin_button_new_with_range ((double) (-100), (double) 200, (double) 1));
			_tmp11_ = gconf_client_get_int (self->priv->_default_client, entry->key, &_inner_error_);
			if (_inner_error_ != NULL) {
				g_propagate_error (error, _inner_error_);
				(box == NULL) ? NULL : (box = (g_object_unref (box), NULL));
				(spin == NULL) ? NULL : (spin = (g_object_unref (spin), NULL));
				(entry == NULL) ? NULL : (entry = (gconf_entry_unref (entry), NULL));
				(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
				(row == NULL) ? NULL : (row = (g_object_unref (row), NULL));
				(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
				(action_widget == NULL) ? NULL : (action_widget = (g_object_unref (action_widget), NULL));
				(render_widget == NULL) ? NULL : (render_widget = (g_object_unref (render_widget), NULL));
				return NULL;
			}
			gtk_spin_button_set_value (spin, (double) _tmp11_);
			g_signal_connect_object (spin, "value-changed", (GCallback) _gconf_dialog_onSpinButtonValueChanged_gtk_spin_button_value_changed, self, 0);
			gtk_box_pack_start ((GtkBox*) box, (GtkWidget*) label, FALSE, FALSE, (guint) 2);
			gtk_box_pack_start ((GtkBox*) box, (GtkWidget*) spin, FALSE, FALSE, (guint) 2);
			_tmp13_ = NULL;
			_tmp12_ = NULL;
			action_widget = (_tmp13_ = (_tmp12_ = (GtkWidget*) spin, (_tmp12_ == NULL) ? NULL : g_object_ref (_tmp12_)), (action_widget == NULL) ? NULL : (action_widget = (g_object_unref (action_widget), NULL)), _tmp13_);
			_tmp15_ = NULL;
			_tmp14_ = NULL;
			render_widget = (_tmp15_ = (_tmp14_ = (GtkWidget*) box, (_tmp14_ == NULL) ? NULL : g_object_ref (_tmp14_)), (render_widget == NULL) ? NULL : (render_widget = (g_object_unref (render_widget), NULL)), _tmp15_);
			(box == NULL) ? NULL : (box = (g_object_unref (box), NULL));
			(spin == NULL) ? NULL : (spin = (g_object_unref (spin), NULL));
			break;
		}
		default:
		{
			GtkWidget* _tmp16_;
			_tmp16_ = NULL;
			return (_tmp16_ = (GtkWidget*) g_object_ref_sink ((GtkEventBox*) gtk_event_box_new ()), (entry == NULL) ? NULL : (entry = (gconf_entry_unref (entry), NULL)), (schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL)), (row == NULL) ? NULL : (row = (g_object_unref (row), NULL)), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), (action_widget == NULL) ? NULL : (action_widget = (g_object_unref (action_widget), NULL)), (render_widget == NULL) ? NULL : (render_widget = (g_object_unref (render_widget), NULL)), _tmp16_);
		}
	}
	gtk_widget_set_tooltip_text (action_widget, tooltip);
	g_object_set_data ((GObject*) action_widget, "gconf-entry", entry);
	g_object_set_data ((GObject*) action_widget, "gconf-schema", schema);
	reset = g_object_ref_sink ((GtkButton*) gtk_button_new_from_stock (GTK_STOCK_CLEAR));
	gtk_widget_set_tooltip_text ((GtkWidget*) reset, _ ("Reset to the default value"));
	/*reset.set_image(new Gtk.Image.from_stock("gtk-clear", Gtk.IconSize.SMALL_TOOLBAR));*/
	g_object_set_data ((GObject*) reset, "gconf-entry", entry);
	g_object_set_data ((GObject*) reset, "gconf-schema", schema);
	g_object_set_data ((GObject*) reset, "target", action_widget);
	g_signal_connect_object (reset, "clicked", (GCallback) _gconf_dialog_onResetButtonPressed_gtk_button_clicked, self, 0);
	gtk_box_pack_start (row, render_widget, FALSE, FALSE, (guint) 2);
	gtk_box_pack_end (row, (GtkWidget*) reset, FALSE, FALSE, (guint) 2);
	_tmp17_ = NULL;
	return (_tmp17_ = (GtkWidget*) row, (entry == NULL) ? NULL : (entry = (gconf_entry_unref (entry), NULL)), (schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL)), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), (action_widget == NULL) ? NULL : (action_widget = (g_object_unref (action_widget), NULL)), (render_widget == NULL) ? NULL : (render_widget = (g_object_unref (render_widget), NULL)), (reset == NULL) ? NULL : (reset = (g_object_unref (reset), NULL)), _tmp17_);
}


static void gconf_dialog_onCheckButtonActivated (GConfDialog* self, GtkCheckButton* widget) {
	GError * _inner_error_;
	GConfEntry* _tmp0_;
	GConfEntry* entry;
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	_inner_error_ = NULL;
	_tmp0_ = NULL;
	entry = (_tmp0_ = (GConfEntry*) g_object_get_data ((GObject*) widget, "gconf-entry"), (_tmp0_ == NULL) ? NULL : gconf_entry_ref (_tmp0_), _tmp0_);
	{
		gconf_client_set_bool (self->priv->_default_client, entry->key, gtk_toggle_button_get_active ((GtkToggleButton*) widget), &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch7_g_error;
			goto __finally7;
		}
	}
	goto __finally7;
	__catch7_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			g_warning ("gtkextra-gconfdialog.vala:130: %s", e->message);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally7:
	if (_inner_error_ != NULL) {
		(entry == NULL) ? NULL : (entry = (gconf_entry_unref (entry), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	(entry == NULL) ? NULL : (entry = (gconf_entry_unref (entry), NULL));
}


static void gconf_dialog_onSpinButtonValueChanged (GConfDialog* self, GtkSpinButton* widget) {
	GError * _inner_error_;
	GConfEntry* _tmp0_;
	GConfEntry* entry;
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	_inner_error_ = NULL;
	_tmp0_ = NULL;
	entry = (_tmp0_ = (GConfEntry*) g_object_get_data ((GObject*) widget, "gconf-entry"), (_tmp0_ == NULL) ? NULL : gconf_entry_ref (_tmp0_), _tmp0_);
	{
		gconf_client_set_int (self->priv->_default_client, entry->key, (gint) gtk_spin_button_get_value (widget), &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch8_g_error;
			goto __finally8;
		}
	}
	goto __finally8;
	__catch8_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			g_warning ("gtkextra-gconfdialog.vala:139: %s", e->message);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally8:
	if (_inner_error_ != NULL) {
		(entry == NULL) ? NULL : (entry = (gconf_entry_unref (entry), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	(entry == NULL) ? NULL : (entry = (gconf_entry_unref (entry), NULL));
}


/**
 * Safely obtain a schema for the entry.
 * If no schema is found, return a fake schema.
 */
static GConfSchema* gconf_dialog_safely_get_schema (GConfDialog* self, GConfEntry* entry) {
	GError * _inner_error_;
	const char* schema_name;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (entry != NULL, NULL);
	_inner_error_ = NULL;
	schema_name = gconf_entry_get_schema_name (entry);
	if (schema_name == NULL) {
		GConfSchema* rt;
		g_critical ("gtkextra-gconfdialog.vala:151: schema not found for entry %s", gconf_entry_get_key (entry));
		rt = gconf_schema_new ();
		gconf_schema_set_short_desc (rt, gconf_entry_get_key (entry));
		gconf_schema_set_long_desc (rt, "This key lacks a schema");
		gconf_schema_set_type (rt, GCONF_VALUE_STRING);
		return rt;
	} else {
		GConfSchema* _tmp1_;
		_tmp1_ = gconf_client_get_schema (self->priv->_default_client, schema_name, &_inner_error_);
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return NULL;
		}
		return _tmp1_;
	}
}


static void gconf_dialog_onResetButtonPressed (GConfDialog* self, GtkButton* widget) {
	GConfSchema* schema;
	GConfValue* default_value;
	GtkWidget* _tmp0_;
	GtkWidget* target;
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	schema = (GConfSchema*) g_object_get_data ((GObject*) widget, "gconf-schema");
	default_value = gconf_schema_get_default_value (schema);
	_tmp0_ = NULL;
	target = (_tmp0_ = GTK_WIDGET (g_object_get_data ((GObject*) widget, "target")), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	switch (gconf_schema_get_type (schema)) {
		case GCONF_VALUE_BOOL:
		{
			GtkCheckButton* _tmp2_;
			GtkWidget* _tmp1_;
			GtkCheckButton* checkbutton;
			_tmp2_ = NULL;
			_tmp1_ = NULL;
			checkbutton = (_tmp2_ = (_tmp1_ = target, GTK_IS_CHECK_BUTTON (_tmp1_) ? ((GtkCheckButton*) _tmp1_) : NULL), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
			gtk_toggle_button_set_active ((GtkToggleButton*) checkbutton, gconf_value_get_bool (default_value));
			(checkbutton == NULL) ? NULL : (checkbutton = (g_object_unref (checkbutton), NULL));
			break;
		}
		case GCONF_VALUE_STRING:
		{
			GtkEntry* _tmp4_;
			GtkWidget* _tmp3_;
			GtkEntry* entrybox;
			_tmp4_ = NULL;
			_tmp3_ = NULL;
			entrybox = (_tmp4_ = (_tmp3_ = target, GTK_IS_ENTRY (_tmp3_) ? ((GtkEntry*) _tmp3_) : NULL), (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_));
			gtk_entry_set_text (entrybox, gconf_value_get_string (default_value));
			(entrybox == NULL) ? NULL : (entrybox = (g_object_unref (entrybox), NULL));
			break;
		}
		case GCONF_VALUE_INT:
		{
			GtkSpinButton* _tmp6_;
			GtkWidget* _tmp5_;
			GtkSpinButton* spin;
			_tmp6_ = NULL;
			_tmp5_ = NULL;
			spin = (_tmp6_ = (_tmp5_ = target, GTK_IS_SPIN_BUTTON (_tmp5_) ? ((GtkSpinButton*) _tmp5_) : NULL), (_tmp6_ == NULL) ? NULL : g_object_ref (_tmp6_));
			gtk_spin_button_set_value (spin, (double) gconf_value_get_int (default_value));
			(spin == NULL) ? NULL : (spin = (g_object_unref (spin), NULL));
			break;
		}
	}
	(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
}


static GObject * gconf_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GConfDialogClass * klass;
	GObjectClass * parent_class;
	GConfDialog * self;
	klass = GCONF_DIALOG_CLASS (g_type_class_peek (TYPE_GCONF_DIALOG));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = GCONF_DIALOG (obj);
	{
		GConfClient* _tmp1_;
		GConfClient* _tmp0_;
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		self->priv->_default_client = (_tmp1_ = (_tmp0_ = gconf_client_get_default (), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_default_client == NULL) ? NULL : (self->priv->_default_client = (g_object_unref (self->priv->_default_client), NULL)), _tmp1_);
		/*vbox.width_request = 320;*/
		gtk_dialog_add_button ((GtkDialog*) self, GTK_STOCK_CLOSE, (gint) GTK_RESPONSE_CLOSE);
		gtk_dialog_add_button ((GtkDialog*) self, GTK_STOCK_HELP, (gint) GTK_RESPONSE_HELP);
	}
	return obj;
}


static void gconf_dialog_class_init (GConfDialogClass * klass) {
	gconf_dialog_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GConfDialogPrivate));
	G_OBJECT_CLASS (klass)->constructor = gconf_dialog_constructor;
	G_OBJECT_CLASS (klass)->finalize = gconf_dialog_finalize;
}


static void gconf_dialog_instance_init (GConfDialog * self) {
	self->priv = GCONF_DIALOG_GET_PRIVATE (self);
}


static void gconf_dialog_finalize (GObject* obj) {
	GConfDialog * self;
	self = GCONF_DIALOG (obj);
	(self->priv->_default_client == NULL) ? NULL : (self->priv->_default_client = (g_object_unref (self->priv->_default_client), NULL));
	G_OBJECT_CLASS (gconf_dialog_parent_class)->finalize (obj);
}


GType gconf_dialog_get_type (void) {
	static GType gconf_dialog_type_id = 0;
	if (gconf_dialog_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GConfDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gconf_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GConfDialog), 0, (GInstanceInitFunc) gconf_dialog_instance_init, NULL };
		gconf_dialog_type_id = g_type_register_static (GTK_TYPE_DIALOG, "GConfDialog", &g_define_type_info, 0);
	}
	return gconf_dialog_type_id;
}




